@@ -25,7 +25,7 @@ import numpy as np
 import pandas as pd
 
 from .statistics.core import StatsError
-from .statistics.descriptive import get_maf
+from .statistics.descriptive import get_maf, get_sex_maf
 from . import subscribers as subscribers_module
 from .modelspec import SNPs, PheWAS, modelspec_from_formula
 
@@ -66,11 +66,17 @@ def _generate_sample_order(x_samples, geno_samples):
 
 
 def _gwas_worker(q, results_q, failed, abort, fit, y, X, samples, maf_t=None,
-                 interaction=None):
+                 interaction=None, sample_sex=None):
     # The sample order (to add genotypes to the X data frame
     geno_index = None
     sample_order = None
 
+    # Checking if we have duplicated samples (i.e. for mixedlm)
+    duplicated = X.index.duplicated(keep="first")
+    has_duplicates = duplicated.any()
+    if has_duplicates:
+        unique_samples = ~duplicated
+
     # Get a SNP
     while not abort.is_set():
         # Get a SNP from the Queue.
@@ -102,18 +108,14 @@ def _gwas_worker(q, results_q, failed, abort, fit, y, X, samples, maf_t=None,
         # Set all to missing genotypes
         X.loc[:, "SNPs"] = np.nan
 
+        # If interaction, we remove the previously computed columns
+        if interaction:
+            X = X.drop(list(interaction.keys()), axis=1, errors="ignore")
+
         # Set the genotypes
         X.loc[sample_order, "SNPs"] = snp.genotypes[geno_index]
 
-        if interaction:
-            # We have an interaction with SNPs, so we also need to compute it
-            # by multiplying SNPs with every columns
-            for key, cols in interaction.items():
-                X.loc[:, key] = functools.reduce(
-                    np.multiply,
-                    (X[col] for col in itertools.chain(["SNPs"], cols)),
-                )
-
+        # The not missing values
         not_missing = _missing(y, X)
 
         if np.sum(not_missing) == 0:
@@ -122,11 +124,28 @@ def _gwas_worker(q, results_q, failed, abort, fit, y, X, samples, maf_t=None,
             continue
 
         # Computing MAF
-        maf, minor, major, flip = get_maf(
-            genotypes=X.loc[not_missing, "SNPs"],
-            minor=snp.coded,
-            major=snp.reference,
-        )
+        samples_for_maf = not_missing
+        if has_duplicates:
+            samples_for_maf = samples_for_maf & unique_samples
+
+        maf = None
+        minor = None
+        major = None
+        flip = None
+        if sample_sex is None:
+            maf, minor, major, flip = get_maf(
+                genotypes=X.loc[samples_for_maf, "SNPs"],
+                minor=snp.coded,
+                major=snp.reference,
+            )
+        else:
+            indexes = X.index[samples_for_maf]
+            maf, minor, major, flip = get_sex_maf(
+                genotypes=X.loc[indexes, "SNPs"],
+                minor=snp.coded,
+                major=snp.reference,
+                sexes=sample_sex.reindex(indexes),
+            )
 
         # Is the MAF below the threshold?
         if maf_t is not None and maf < maf_t:
@@ -137,6 +156,15 @@ def _gwas_worker(q, results_q, failed, abort, fit, y, X, samples, maf_t=None,
         if flip:
             X.loc[:, "SNPs"] = 2 - X.loc[:, "SNPs"]
 
+        if interaction:
+            # We have an interaction with SNPs, so we also need to compute it
+            # by multiplying SNPs with every columns
+            for key, cols in interaction.items():
+                X.loc[:, key] = functools.reduce(
+                    np.multiply,
+                    (X[col] for col in itertools.chain(["SNPs"], cols)),
+                )
+
         # Computing
         results = None
         try:
@@ -188,19 +216,20 @@ def _log_warnings(identifier, warning_list):
 
 def execute_formula(phenotypes, genotypes, formula, test, test_kwargs=None,
                     subscribers=None, variant_predicates=None,
-                    output_prefix=None, maf_t=None, cpus=None):
+                    output_prefix=None, maf_t=None, cpus=None,
+                    sexual_chromosome=False):
 
     # Getting the model specification and the subgroups (if any)
     modelspec, subgroups = modelspec_from_formula(formula, test, test_kwargs)
 
     # Executing
     execute(phenotypes, genotypes, modelspec, subscribers, variant_predicates,
-            output_prefix, subgroups, maf_t, cpus)
+            output_prefix, subgroups, maf_t, cpus, sexual_chromosome)
 
 
 def execute(phenotypes, genotypes, modelspec, subscribers=None,
             variant_predicates=None, output_prefix=None, subgroups=None,
-            maf_t=None, cpus=None):
+            maf_t=None, cpus=None, sexual_chromosome=False):
     """Execute an analysis.
 
     Args:
@@ -213,6 +242,7 @@ def execute(phenotypes, genotypes, modelspec, subscribers=None,
         subgroups (list): The subgroup analysis.
         maf_t (float): The MAF threshold.
         cpus (int): The number of CPUs to perform the analysis.
+        sexual_chromosome (bool): The analysis is performed on sexual chrom.
 
     """
     if subscribers is None:
@@ -260,13 +290,22 @@ def execute(phenotypes, genotypes, modelspec, subscribers=None,
         "failed": []
     }
 
+    # If sexual chromosomes, we need to get the sex from the phenotypes
+    sample_sex = None
+    if sexual_chromosome:
+        logger.info("Analysis is performed on a sexual chromosome")
+        sample_sex = phenotypes.get_sex()
+        logger.info("  - {:,d} males".format((sample_sex == 1).sum()))
+        logger.info("  - {:,d} females".format((sample_sex == 0).sum()))
+        logger.info("  - {:,d} unknowns".format((sample_sex.isnull().sum())))
+
     if modelspec.stratify_by is not None:
         _execute_stratified(genotypes, modelspec, subscribers, y, X,
                             variant_predicates, subgroups, messages, maf_t,
-                            cpus)
+                            cpus, sample_sex)
     elif SNPs in modelspec.predictors:
         _execute_gwas(genotypes, modelspec, subscribers, y, X,
-                      variant_predicates, messages, maf_t, cpus)
+                      variant_predicates, messages, maf_t, cpus, sample_sex)
     else:
         _execute_simple(modelspec, subscribers, y, X, variant_predicates,
                         messages)
@@ -413,7 +452,7 @@ def _phewas_worker(data, predictors, abort, fit, phen_queue, results_queue):
 
 def _execute_stratified(genotypes, modelspec, subscribers, y, X,
                         variant_predicates, subgroups, messages, maf_t=None,
-                        cpus=None):
+                        cpus=None, sample_sex=None):
     # Levels.
     assert len(modelspec.stratify_by) == len(subgroups)
 
@@ -495,7 +534,7 @@ def _execute_stratified(genotypes, modelspec, subscribers, y, X,
         if gwas_mode:
             _execute_gwas(
                 genotypes, modelspec, subscribers, y.loc[idx, :], this_x,
-                variant_predicates, messages, maf_t, cpus,
+                variant_predicates, messages, maf_t, cpus, sample_sex,
             )
         else:
             _execute_simple(
@@ -552,7 +591,7 @@ def _execute_simple(modelspec, subscribers, y, X, variant_predicates,
 
 
 def _execute_gwas(genotypes, modelspec, subscribers, y, X, variant_predicates,
-                  messages, maf_t=None, cpus=None):
+                  messages, maf_t=None, cpus=None, sample_sex=None):
         if cpus is None:
             cpus = multiprocessing.cpu_count() - 1
 
@@ -586,7 +625,8 @@ def _execute_gwas(genotypes, modelspec, subscribers, y, X, variant_predicates,
             worker = multiprocessing.Process(
                 target=_gwas_worker,
                 args=(q, results, failed, abort, fit, this_y, this_X, samples,
-                      maf_t, gwas_interaction)
+                      maf_t, gwas_interaction,
+                      sample_sex if sample_sex is None else sample_sex.copy())
             )
 
             workers.append(worker)
@@ -669,7 +709,12 @@ def _execute_gwas(genotypes, modelspec, subscribers, y, X, variant_predicates,
         for snp in not_analyzed:
             messages["skipped"].append(snp)
 
-        # Sanity check that there is nothing important left in the queues.
+        # Join the workers.
+        for worker in workers:
+            worker.join()
+
+        # Sanity check that there is nothing important left in the queues and
+        # join the queues
         queues_iter = zip(
             ('results', 'failed', 'q'), (results, failed, q)
         )
@@ -680,8 +725,4 @@ def _execute_gwas(genotypes, modelspec, subscribers, y, X, variant_predicates,
                 a_queue.task_done()
             a_queue.join()
 
-        # Join the workers.
-        for worker in workers:
-            worker.join()
-
         logger.info("Analysis completed")
