@@ -52,7 +52,7 @@ class TestStatsMixedLM(unittest.TestCase):
         )
 
         # Permuting the sample to add a bit of randomness
-        new_sample_order = np.random.permutation(data.index)
+        new_sample_order = np.random.permutation(np.copy(data.index.values))
 
         # Creating the genotypes data frame
         genotypes = data.loc[
@@ -90,6 +90,7 @@ class TestStatsMixedLM(unittest.TestCase):
             np.random.permutation(self.phenotypes.data.shape[0]),
             np.random.permutation(self.phenotypes.data.shape[1])
         ]
+        self.phenotypes._repeated = True
 
     @classmethod
     def tearDownClass(cls):
@@ -128,6 +129,8 @@ class TestStatsMixedLM(unittest.TestCase):
         # Checking the first marker (snp1)
         results = gwas_results["snp1"]
         self.assertEqual("snp1", results["SNPs"]["name"])
+        self.assertEqual(180, results["MODEL"]["nobs"])
+        self.assertEqual(60, results["MODEL"]["n_groups"])
         self.assertEqual("3", results["SNPs"]["chrom"])
         self.assertEqual(1234, results["SNPs"]["pos"])
         self.assertEqual("T", results["SNPs"]["minor"])
@@ -150,6 +153,8 @@ class TestStatsMixedLM(unittest.TestCase):
         # Checking the second marker (snp2)
         results = gwas_results["snp2"]
         self.assertEqual("snp2", results["SNPs"]["name"])
+        self.assertEqual(180, results["MODEL"]["nobs"])
+        self.assertEqual(60, results["MODEL"]["n_groups"])
         self.assertEqual("3", results["SNPs"]["chrom"])
         self.assertEqual(2345, results["SNPs"]["pos"])
         self.assertEqual("C", results["SNPs"]["minor"])
@@ -172,6 +177,8 @@ class TestStatsMixedLM(unittest.TestCase):
         # Checking the third marker (snp3)
         results = gwas_results["snp3"]
         self.assertEqual("snp3", results["SNPs"]["name"])
+        self.assertEqual(180, results["MODEL"]["nobs"])
+        self.assertEqual(60, results["MODEL"]["n_groups"])
         self.assertEqual("2", results["SNPs"]["chrom"])
         self.assertEqual(3456, results["SNPs"]["pos"])
         self.assertEqual("G", results["SNPs"]["minor"])
@@ -194,6 +201,8 @@ class TestStatsMixedLM(unittest.TestCase):
         # Checking the fourth marker (snp4)
         results = gwas_results["snp4"]
         self.assertEqual("snp4", results["SNPs"]["name"])
+        self.assertEqual(180, results["MODEL"]["nobs"])
+        self.assertEqual(60, results["MODEL"]["n_groups"])
         self.assertEqual("22", results["SNPs"]["chrom"])
         self.assertEqual(4567, results["SNPs"]["pos"])
         self.assertEqual("A", results["SNPs"]["minor"])
@@ -248,6 +257,8 @@ class TestStatsMixedLM(unittest.TestCase):
         # Checking the first marker (snp1)
         results = gwas_results["snp1"]
         self.assertEqual("snp1", results["SNPs"]["name"])
+        self.assertEqual(180, results["MODEL"]["nobs"])
+        self.assertEqual(60, results["MODEL"]["n_groups"])
         self.assertEqual("3", results["SNPs"]["chrom"])
         self.assertEqual(1234, results["SNPs"]["pos"])
         self.assertEqual("T", results["SNPs"]["minor"])
@@ -271,6 +282,8 @@ class TestStatsMixedLM(unittest.TestCase):
         # Checking the second marker (snp2)
         results = gwas_results["snp2"]
         self.assertEqual("snp2", results["SNPs"]["name"])
+        self.assertEqual(180, results["MODEL"]["nobs"])
+        self.assertEqual(60, results["MODEL"]["n_groups"])
         self.assertEqual("3", results["SNPs"]["chrom"])
         self.assertEqual(2345, results["SNPs"]["pos"])
         self.assertEqual("C", results["SNPs"]["minor"])
@@ -294,6 +307,8 @@ class TestStatsMixedLM(unittest.TestCase):
         # Checking the third marker (snp3)
         results = gwas_results["snp3"]
         self.assertEqual("snp3", results["SNPs"]["name"])
+        self.assertEqual(180, results["MODEL"]["nobs"])
+        self.assertEqual(60, results["MODEL"]["n_groups"])
         self.assertEqual("2", results["SNPs"]["chrom"])
         self.assertEqual(3456, results["SNPs"]["pos"])
         self.assertEqual("G", results["SNPs"]["minor"])
@@ -317,6 +332,8 @@ class TestStatsMixedLM(unittest.TestCase):
         # Checking the fourth marker (snp4)
         results = gwas_results["snp4"]
         self.assertEqual("snp4", results["SNPs"]["name"])
+        self.assertEqual(180, results["MODEL"]["nobs"])
+        self.assertEqual(60, results["MODEL"]["n_groups"])
         self.assertEqual("22", results["SNPs"]["chrom"])
         self.assertEqual(4567, results["SNPs"]["pos"])
         self.assertEqual("A", results["SNPs"]["minor"])
@@ -365,6 +382,8 @@ class TestStatsMixedLM(unittest.TestCase):
 
         # Checking the marker information
         self.assertEqual("snp1", results["snp1"]["name"])
+        self.assertEqual(180, results["MODEL"]["nobs"])
+        self.assertEqual(60, results["MODEL"]["n_groups"])
         self.assertEqual("3", results["snp1"]["chrom"])
         self.assertEqual(1234, results["snp1"]["pos"])
         self.assertEqual("T", results["snp1"]["minor"])
@@ -414,6 +433,8 @@ class TestStatsMixedLM(unittest.TestCase):
 
         # Checking the marker information
         self.assertEqual("snp1", results["snp1"]["name"])
+        self.assertEqual(180, results["MODEL"]["nobs"])
+        self.assertEqual(60, results["MODEL"]["n_groups"])
         self.assertEqual("3", results["snp1"]["chrom"])
         self.assertEqual(1234, results["snp1"]["pos"])
         self.assertEqual("T", results["snp1"]["minor"])
@@ -466,6 +487,8 @@ class TestStatsMixedLM(unittest.TestCase):
 
         # Checking the marker information
         self.assertEqual("snp1", results["snp1"]["name"])
+        self.assertEqual(180, results["MODEL"]["nobs"])
+        self.assertEqual(60, results["MODEL"]["n_groups"])
         self.assertEqual("3", results["snp1"]["chrom"])
         self.assertEqual(1234, results["snp1"]["pos"])
         self.assertEqual("T", results["snp1"]["minor"])
@@ -519,6 +542,8 @@ class TestStatsMixedLM(unittest.TestCase):
 
         # Checking the marker information
         self.assertEqual("snp1", results["snp1"]["name"])
+        self.assertEqual(180, results["MODEL"]["nobs"])
+        self.assertEqual(60, results["MODEL"]["n_groups"])
         self.assertEqual("3", results["snp1"]["chrom"])
         self.assertEqual(1234, results["snp1"]["pos"])
         self.assertEqual("T", results["snp1"]["minor"])
@@ -569,6 +594,8 @@ class TestStatsMixedLM(unittest.TestCase):
 
         # Checking the marker information
         self.assertEqual("snp2", results["snp2"]["name"])
+        self.assertEqual(180, results["MODEL"]["nobs"])
+        self.assertEqual(60, results["MODEL"]["n_groups"])
         self.assertEqual("3", results["snp2"]["chrom"])
         self.assertEqual(2345, results["snp2"]["pos"])
         self.assertEqual("C", results["snp2"]["minor"])
@@ -616,6 +643,16 @@ class TestStatsMixedLM(unittest.TestCase):
         )
         results = subscriber.results[0]
 
+        # Checking the marker information
+        self.assertEqual("snp2", results["snp2"]["name"])
+        self.assertEqual(180, results["MODEL"]["nobs"])
+        self.assertEqual(60, results["MODEL"]["n_groups"])
+        self.assertEqual("3", results["snp2"]["chrom"])
+        self.assertEqual(2345, results["snp2"]["pos"])
+        self.assertEqual("C", results["snp2"]["minor"])
+        self.assertEqual("A", results["snp2"]["major"])
+        self.assertAlmostEqual(0.36666666666666664, results["snp2"]["maf"])
+
         # Checking the marker statistics (according to R lme4)
         self.assertAlmostEqual(28.87619649885465023, results["snp2"]["coef"])
         self.assertAlmostEqual(7.2921675410708389, results["snp2"]["std_err"],
@@ -662,6 +699,8 @@ class TestStatsMixedLM(unittest.TestCase):
 
         # Checking the marker information
         self.assertEqual("snp2", results["snp2"]["name"])
+        self.assertEqual(180, results["MODEL"]["nobs"])
+        self.assertEqual(60, results["MODEL"]["n_groups"])
         self.assertEqual("3", results["snp2"]["chrom"])
         self.assertEqual(2345, results["snp2"]["pos"])
         self.assertEqual("C", results["snp2"]["minor"])
@@ -715,6 +754,8 @@ class TestStatsMixedLM(unittest.TestCase):
 
         # Checking the marker information
         self.assertEqual("snp2", results["snp2"]["name"])
+        self.assertEqual(180, results["MODEL"]["nobs"])
+        self.assertEqual(60, results["MODEL"]["n_groups"])
         self.assertEqual("3", results["snp2"]["chrom"])
         self.assertEqual(2345, results["snp2"]["pos"])
         self.assertEqual("C", results["snp2"]["minor"])
@@ -765,6 +806,8 @@ class TestStatsMixedLM(unittest.TestCase):
 
         # Checking the marker information
         self.assertEqual("snp3", results["snp3"]["name"])
+        self.assertEqual(180, results["MODEL"]["nobs"])
+        self.assertEqual(60, results["MODEL"]["n_groups"])
         self.assertEqual("2", results["snp3"]["chrom"])
         self.assertEqual(3456, results["snp3"]["pos"])
         self.assertEqual("G", results["snp3"]["minor"])
@@ -814,6 +857,8 @@ class TestStatsMixedLM(unittest.TestCase):
 
         # Checking the marker information
         self.assertEqual("snp3", results["snp3"]["name"])
+        self.assertEqual(180, results["MODEL"]["nobs"])
+        self.assertEqual(60, results["MODEL"]["n_groups"])
         self.assertEqual("2", results["snp3"]["chrom"])
         self.assertEqual(3456, results["snp3"]["pos"])
         self.assertEqual("G", results["snp3"]["minor"])
@@ -866,6 +911,8 @@ class TestStatsMixedLM(unittest.TestCase):
 
         # Checking the marker information
         self.assertEqual("snp3", results["snp3"]["name"])
+        self.assertEqual(180, results["MODEL"]["nobs"])
+        self.assertEqual(60, results["MODEL"]["n_groups"])
         self.assertEqual("2", results["snp3"]["chrom"])
         self.assertEqual(3456, results["snp3"]["pos"])
         self.assertEqual("G", results["snp3"]["minor"])
@@ -919,6 +966,8 @@ class TestStatsMixedLM(unittest.TestCase):
 
         # Checking the marker information
         self.assertEqual("snp3", results["snp3"]["name"])
+        self.assertEqual(180, results["MODEL"]["nobs"])
+        self.assertEqual(60, results["MODEL"]["n_groups"])
         self.assertEqual("2", results["snp3"]["chrom"])
         self.assertEqual(3456, results["snp3"]["pos"])
         self.assertEqual("G", results["snp3"]["minor"])
@@ -969,6 +1018,8 @@ class TestStatsMixedLM(unittest.TestCase):
 
         # Checking the marker information
         self.assertEqual("snp4", results["snp4"]["name"])
+        self.assertEqual(180, results["MODEL"]["nobs"])
+        self.assertEqual(60, results["MODEL"]["n_groups"])
         self.assertEqual("22", results["snp4"]["chrom"])
         self.assertEqual(4567, results["snp4"]["pos"])
         self.assertEqual("A", results["snp4"]["minor"])
@@ -1018,6 +1069,8 @@ class TestStatsMixedLM(unittest.TestCase):
 
         # Checking the marker information
         self.assertEqual("snp4", results["snp4"]["name"])
+        self.assertEqual(180, results["MODEL"]["nobs"])
+        self.assertEqual(60, results["MODEL"]["n_groups"])
         self.assertEqual("22", results["snp4"]["chrom"])
         self.assertEqual(4567, results["snp4"]["pos"])
         self.assertEqual("A", results["snp4"]["minor"])
@@ -1070,6 +1123,8 @@ class TestStatsMixedLM(unittest.TestCase):
 
         # Checking the marker information
         self.assertEqual("snp4", results["snp4"]["name"])
+        self.assertEqual(180, results["MODEL"]["nobs"])
+        self.assertEqual(60, results["MODEL"]["n_groups"])
         self.assertEqual("22", results["snp4"]["chrom"])
         self.assertEqual(4567, results["snp4"]["pos"])
         self.assertEqual("A", results["snp4"]["minor"])
@@ -1123,6 +1178,8 @@ class TestStatsMixedLM(unittest.TestCase):
 
         # Checking the marker information
         self.assertEqual("snp4", results["snp4"]["name"])
+        self.assertEqual(180, results["MODEL"]["nobs"])
+        self.assertEqual(60, results["MODEL"]["n_groups"])
         self.assertEqual("22", results["snp4"]["chrom"])
         self.assertEqual(4567, results["snp4"]["pos"])
         self.assertEqual("A", results["snp4"]["minor"])
